use std::{collections::HashMap, str::FromStr, sync::{mpsc::{self, Receiver, Sender}, Arc, Mutex, MutexGuard}, time::Duration};
use downloader::song::SongStatus;
use log::warn;
use xmpd_manifest::song::Song;

pub mod downloader;

type Result<T> = anyhow::Result<T>;

lazy_static::lazy_static!(
    static ref CACHE: Arc<Mutex<Cache>> = Arc::new(Mutex::new(Cache::default()));
);

#[derive(Debug, Default)]
pub struct Cache {
    cache_dir: camino::Utf8PathBuf,
    song_cache: HashMap<uuid::Uuid, DlStatus>,
    queue: Vec<(uuid::Uuid, Song)>
    // TODO: Add Icon, metadata cache
}

#[derive(Debug, Clone)]
pub enum DlStatus {
    Done(camino::Utf8PathBuf),
    Downloading,
    Error(String),
}

#[derive(Debug, Clone)]
pub enum Message {
    DownloadDone(uuid::Uuid),
}

impl Cache {
    pub fn get() -> crate::Result<MutexGuard<'static, Self>> {
        match CACHE.lock() {
            Ok(l) => Ok(l),
            Err(e) => Err(anyhow::anyhow!(format!("{e:?}"))),
        }
    }

    pub fn init(&mut self) -> Result<Receiver<Message>> {
        let (internal_tx, cache_rx) = mpsc::channel::<Message>();
        // let (internal_rx, cache_tx) = mpsc::channel::<Message>();
        start_cache_mv_thread(internal_tx);
        self.cache_dir = xmpd_cliargs::CLIARGS.cache_path();
        
        { // Get cached songs
            let mut song_cache_dir = self.cache_dir.clone();
            std::fs::create_dir_all(&song_cache_dir)?;
            song_cache_dir.push("songs");
            for file in song_cache_dir.read_dir_utf8()? {
                if let Ok(file) = file {
                    if !file.file_type()?.is_file() {
                        log::warn!("Non song file in: {}", file.path());
                        continue;
                    }
                    let file_path = file.path();
                    let file2 = file_path.with_extension("");
                    if let Some(file_name) = file2.file_name() {
                        let id = uuid::Uuid::from_str(file_name)?;
                        log::debug!("Found song {id}");
                        // TODO: Check if id is in manifest
                        self.song_cache.insert(id, DlStatus::Done(file_path.to_path_buf()));
                    }
                    
                }
            }
        }
        { // Get cached icons
        }
        { // Get Cached meta
        }
        Ok(cache_rx)
    }

    pub fn download_to_cache(&mut self, sid: uuid::Uuid, song: Song) {
        self.queue.push((sid, song));
        self.song_cache.insert(sid, DlStatus::Downloading);
    }

    pub fn download_to_cache_multiple(&mut self, mut songs: Vec<(uuid::Uuid, Song)>) {
        while let Some((sid, song)) = songs.pop() {
            self.download_to_cache(sid, song);
        }
    }

    pub fn get_cached_song_status(&mut self, sid: &uuid::Uuid) -> Option<DlStatus> {
        let original = self.song_cache.get(sid)?.clone();
        Some(original)
    }
}

fn start_cache_mv_thread(tx: Sender<Message>) {
    std::thread::spawn(move || {
        loop {
            {
                std::thread::sleep(Duration::from_millis(500));
                let song_format = xmpd_settings::Settings::get().unwrap().tooling.song_format.clone();
                let mut done_jobs = Vec::new();
                let mut dlc = downloader::song::SongCacheDl::get().unwrap();
                for (sid, status) in &dlc.jobs {
                    if *status == SongStatus::Done {
                        let mut cache = CACHE.lock().unwrap();
                        let mut song_p = xmpd_cliargs::CLIARGS.cache_path().clone();
                        song_p.push("songs");
                        song_p.push(sid.clone().to_string());
                        let song_p = song_p.with_extension(&song_format);
                        log::debug!("Found done: {:?}: {}", song_p, song_p.exists());
                        if song_p.exists() {
                            let _ = tx.send(Message::DownloadDone(sid.clone()));
                            cache.song_cache.insert(sid.clone(), DlStatus::Done(song_p)); 
                            done_jobs.push(sid.clone());
                        }
                    }
                }
                for sid in done_jobs {
                    dlc.jobs.remove(&sid);
                }
            }

            {
                let mut cache = Cache::get().unwrap();
                let mut dlc = downloader::song::SongCacheDl::get().unwrap();
                if !dlc.is_job_list_full() {
                    if let Some((sid, song)) = cache.queue.pop() {
                        dlc.download(sid, song).unwrap();
                    }
                }
            }
        }
    });
}
